@@ -1,34 +1,32 @@
 
-import os
-import py
-import pkg_resources
-
-from devpi.util import version as verutil
-from devpi.util import url as urlutil
-from pkg_resources import parse_version
-
-from devpi import log
-import posixpath
+from devpi_common.url import URL
+from devpi_common.metadata import splitbasename, Version, parse_requirement
 
 def out_index(hub, data):
     for name in sorted(data):
         hub.info(name)
 
-def out_project(hub, data, name):
-    versions = list(data)
-    def cmpversion(x, y):
-        return cmp(parse_version(x), parse_version(y))
-
+def out_project(hub, data, req):
     index = hub.current.index[len(hub.current.rooturl):]
-
-    versions.sort(cmp=cmpversion)
-    for version in reversed(versions):
-        #hub.info("%s-%s:" % (name, version))
+    num = 0
+    maxshow = 2
+    for ver in sorted(map(Version, data), reverse=True):
+        version = ver.string
+        if version not in req:
+            continue
+        if num > maxshow and not hub.args.all:
+            num += 1
+            continue
         verdata = data[version]
-        out_project_version_files(hub, verdata, version, index)
+        if out_project_version_files(hub, verdata, version, index):
+            num += 1
         shadowing = data[version].get("+shadowing", [])
         for verdata in shadowing:
-            out_project_version_files(hub, verdata, version, None)
+            if out_project_version_files(hub, verdata, version, None):
+                num += 1
+    if not hub.args.all and num > (maxshow+1):
+        hub.info("%s older versions not shown, use --all to see" %
+                 (num-maxshow-1))
 
 def out_project_version_files(hub, verdata, version, index):
     files = verdata.get("+files")
@@ -44,25 +42,24 @@ def out_project_version_files(hub, verdata, version, index):
             else:
                 hub.line(origin)
             query_file_status(hub, origin)
+    return bool(files)
 
 def query_file_status(hub, origin):
     # XXX this code is not auto-tested in all detail
     # so change with great care or write tests first
     rooturl = hub.current.rooturl
-    res = hub.http_api("get", urlutil.joinpath(rooturl, "/" + origin),
+    res = hub.http_api("get", URL(rooturl, "/" + origin).url,
                        quiet=True)
     assert res.status_code == 200
-    md5 = res["result"].get("md5")
+    md5 = res.result.get("md5")
     if not md5:
         return
-    res = hub.http_api("get", urlutil.joinpath(rooturl,
-                                               "/+tests/%s/toxresult" % md5),
-                       quiet=True)
-    assert res.status_code == 200
-    assert res["type"] == "list:toxresult"
+    res = hub.http_api("get",
+                       URL(rooturl, "/+tests/%s/toxresult" % md5).url,
+                       quiet=True, type="list:toxresult")
     seen = set()
-    for toxresult in reversed(res["result"]):
-        platform = toxresult["platform"]
+    for toxresult in reversed(res.result):
+        toxresult["platform"]
         for envname, env in toxresult["testenvs"].items():
             prefix = "  {host} {platform} {envname}".format(
                      envname=envname, **toxresult)
@@ -111,47 +108,48 @@ def show_commands(hub, commands):
         hub.info("    OK:  %s" % shellcommand)
 
 def main_list(hub, args):
-    current = hub.require_valid_current_with_index()
-    args = hub.args
-
-    hub.info("list result: %s" % (hub.current.index + (args.spec or '')))
-    if not args.spec:
-        data = getjson(hub, None)
-        out_index(hub, data["result"])
+    hub.require_valid_current_with_index()
+    if hub.args.spec:
+        req = parse_requirement(hub.args.spec)
+        url = hub.current.get_project_url(req.project_name)
+        reply = hub.http_api("get", url, type="projectconfig")
+        out_project(hub, reply.result, req)
     else:
-        data = getjson(hub, args.spec)
-        out_project(hub, data["result"], args.spec)
+        reply = hub.http_api("get", hub.current.index,
+                             type="list:projectconfig")
+        out_index(hub, reply.result)
 
 def main_remove(hub, args):
-    current = hub.require_valid_current_with_index()
+    hub.require_valid_current_with_index()
     args = hub.args
-
-    name, ver, suffix = verutil.splitbasename(args.spec)
-    if suffix:
-        hub.fatal("can only delete releases, not single release files")
-    url = hub.current.get_project_url(name)
-    if ver:
-        url = url + ver + "/"
-    data = hub.http_api("get", url)
-    if confirm_delete(hub, data):
-        hub.http_api("delete", url)
-
-def confirm_delete(hub, data):
-    basepath = urlutil.getpath(hub.current.index).lstrip("/")
-    to_delete = []
-    if data["type"] == "projectconfig":
-        for version, verdata in data["result"].items():
-            to_delete.extend(match_release_files(basepath, verdata))
-    elif data["type"] == "versiondata":
-        to_delete.extend(match_release_files(basepath, data["result"]))
-    if not to_delete:
-        hub.line("nothing to delete")
-        return None
+    req = parse_requirement(args.spec)
+    url = hub.current.get_project_url(req.project_name)
+    reply = hub.http_api("get", url, type="projectconfig")
+    ver_to_delete = confirm_delete(hub, reply, req)
+    if ver_to_delete is None:
+        hub.error("not deleting anything")
+        return 1
     else:
-        hub.info("About to remove the following release files and metadata:")
-        for fil in to_delete:
-            hub.info("   " + fil)
-        return hub.ask_confirm("Are you sure")
+        for ver, files in ver_to_delete:
+            hub.info("deleting release %s of %s" % (ver, req.project_name))
+            hub.http_api("delete", url.addpath(ver))
+
+def confirm_delete(hub, reply, req):
+    basepath = URL(hub.current.index).path.lstrip("/")
+    ver_to_delete = []
+    for version, verdata in reply.result.items():
+        if version in req:
+            files_to_delete = list(match_release_files(basepath, verdata))
+            if files_to_delete:  # XXX need to delete metadata without files
+                ver_to_delete.append((version, files_to_delete))
+    if ver_to_delete:
+        hub.info("About to remove the following releases and distributions")
+        for ver, files in ver_to_delete:
+            hub.info("   version: " + ver)
+            for fil in files:
+                hub.info("   - " + fil)
+        if hub.ask_confirm("Are you sure"):
+            return ver_to_delete
 
 def match_release_files(basepath, verdata):
     files = verdata.get("+files", {})
@@ -165,15 +163,6 @@ def match_release_files(basepath, verdata):
 #    if ver in req:
 #        pass
 
-def getjson(hub, path):
-    current = hub.current
-    if not path:
-        check_verify_current(hub)
-        url = current.get_index_url()
-    else:
-        url = urlutil.joinpath(current.get_index_url(), path) + "/"
-    return hub.http_api("get", url, quiet=True)
-
 def check_verify_current(hub):
     if not hub.current.index:
         hub.fatal("cannot use relative path without an active index")
