@@ -5,18 +5,17 @@ import py
 import argparse
 import subprocess
 import devpi
-from devpi.util.lazydecorator import lazydecorator
-from devpi.util import url as urlutil
-from devpi import log, cached_property
-from devpi import __version__ as client_version
+from devpi_common.types import lazydecorator, cached_property
+from devpi_common.url import URL
+from devpi_common.proc import check_output
 from devpi.use import Current
-import requests
+from devpi_common.request import new_requests_session
+from devpi import __version__ as client_version
+
 import json
 std = py.std
 subcommand = lazydecorator()
 
-from devpi import __version__
-
 main_description = """
 The devpi commands (installed via devpi-client) wrap common Python
 packaging, uploading, installation and testing activities, using a remote
@@ -38,22 +37,6 @@ def initmain(argv):
     mod = __import__(mod, None, None, ["__doc__"])
     return Hub(args), getattr(mod, func)
 
-def check_output(*args, **kwargs):
-    from subprocess import Popen, CalledProcessError, PIPE
-    # subprocess.check_output does not exist on python26
-    popen = Popen(stdout=PIPE, *args, **kwargs)
-    output, unused_err = popen.communicate()
-    retcode = popen.poll()
-    if retcode:
-        cmd = kwargs.get("args")
-        if cmd is None:
-            cmd = args[0]
-        if sys.version_info < (2,7) and sys.platform == "win32":
-            raise CalledProcessError(retcode, cmd)
-        else:
-            raise CalledProcessError(retcode, cmd, output=output)
-    return output
-
 notset = object()
 
 class Hub:
@@ -70,7 +53,7 @@ class Hub:
         self.cwd = py.path.local()
         self.quiet = False
         self._last_http_status = None
-        self._session = requests.session()
+        self.http = new_requests_session(agent=("client", client_version))
 
     def set_quiet(self):
         self.quiet = True
@@ -87,15 +70,8 @@ class Hub:
 
     # remote http hooks
 
-    @property
-    def http(self):
-        session = self._session
-        #session.auth = tuple(self.current.auth) if self.current.auth else None
-        session.ConnectionError = requests.exceptions.ConnectionError
-        return session
-
     def http_api(self, method, url, kvdict=None, quiet=False, auth=notset,
-        check_version=True):
+        check_version=True, fatal=True, type=None):
         """ send a json request and return a HTTPReply object which
         adds some extra methods to the requests's Reply object.
 
@@ -104,7 +80,12 @@ class Hub:
 
         This method will not output any information to the user
         if ``quiet`` is set to true.
+
+        If type is specified and the json result type does not match,
+        bail out fatally (unless fatal = False)
         """
+        if isinstance(url, URL):
+            url = url.url
         jsontype = "application/json"
         headers = {"Accept": jsontype, "content-type": jsontype}
         try:
@@ -123,8 +104,8 @@ class Hub:
 
         reply = HTTPReply(r)
 
-        # if we get a 401 it means our auth info is expired or not present
-        if r.status_code == 401:
+        # if we get a 401 with a location, we assume our login data is expired
+        if r.status_code == 401 and "location" in reply.headers:
             if self.current.del_auth():
                 self.error("removed expired authentication information")
 
@@ -133,10 +114,17 @@ class Hub:
 
         if r.status_code in (200, 201):
             # don't show any extra info on success code
+            if type is not None:
+                if reply.type != type:
+                    self.fatal("%s: got result type %r, expected %r" %(
+                                url, reply.type, type))
             return reply
         # feedback reply info to user, possibly bailing out
         if r.status_code >= 400:
-            out = self.fatal
+            if fatal:
+                out = self.fatal
+            else:
+                out = self.error
         elif quiet and not self.args.debug:
             return reply
         else:
@@ -170,19 +158,14 @@ class Hub:
             return self.__workdir
         except AttributeError:
             self.__workdir = py.path.local.make_numbered_dir(prefix="devpi")
-            self.info("created workdir", self.__workdir)
+            self.info("using workdir", self.__workdir)
             return self.__workdir
 
     @cached_property
     def current(self):
         self.clientdir.ensure(dir=1)
         path = self.clientdir.join("current.json")
-        current = Current(path)
-        try:
-            cmd = self.args.mainloc.split(":")[0]
-        except AttributeError:
-            pass
-        return current
+        return Current(path)
 
     @property
     def remoteindex(self):
@@ -193,6 +176,14 @@ class Hub:
             self._remoteindex = RemoteIndex(self.current)
             return self._remoteindex
 
+    def get_existing_file(self, arg):
+        p = py.path.local(arg, expanduser=True)
+        if not p.exists():
+            self.fatal("file does not exist: %s" % p)
+        elif not p.isfile():
+            self.fatal("is not a file: %s" % p)
+        return p
+
     @property
     def path_venvbase(self):
         path = os.environ.get("WORKON_HOME", None)
@@ -200,16 +191,15 @@ class Hub:
             return
         return py.path.local(path)
 
-    def popen_output(self, args, cwd=None):
+    def popen_output(self, args, cwd=None, report=True):
         if isinstance(args, str):
             args = std.shlex.split(args)
         assert args[0], args
         args = [str(x) for x in args]
         if cwd == None:
             cwd = self.cwd
-        self.report_popen(args, cwd)
-        if self.args.dryrun:
-            return
+        if report:
+            self.report_popen(args, cwd)
         return check_output(args, cwd=str(cwd))
 
     def popen(self, args, cwd=None):
@@ -299,6 +289,14 @@ class HTTPReply(object):
     def __getattr__(self, name):
         return getattr(self._response, name)
 
+    @property
+    def type(self):
+        return self._json.get("type")
+
+    @property
+    def result(self):
+        return self._json.get("result")
+
     def json_get(self, jsonkey, default=None):
         r = self._response
         if not r.content or r.headers["content-type"] != "application/json":
@@ -310,7 +308,10 @@ class HTTPReply(object):
 
     @cached_property
     def _json(self):
-        return self._response.json()
+        try:
+            return self._response.json()
+        except ValueError:
+            return {}
 
     def __getitem__(self, name):
         return self._json[name]
@@ -331,15 +332,16 @@ class MyArgumentParser(argparse.ArgumentParser):
 #    pass
 
 def try_argcomplete(parser):
-    try:
-        import argcomplete
-    except ImportError:
-        pass
-    else:
-        argcomplete.autocomplete(parser)
+    if os.environ.get('_ARGCOMPLETE'):
+        try:
+            import argcomplete
+        except ImportError:
+            pass
+        else:
+            argcomplete.autocomplete(parser)
 
 def parse_args(argv):
-    argv = map(str, argv)
+    argv = [str(x) for x in argv]
     parser = getbaseparser(argv[0])
     add_subparsers(parser)
     try_argcomplete(parser)
@@ -429,6 +431,16 @@ def use(parser):
     for installation activities.
     """
 
+    parser.add_argument("--set-cfg", action="store_true", default=None,
+        dest="setcfg",
+        help="create or modify pip/setuptools config files in home directory "
+             "so pip/easy_install will pick up the current devpi index url")
+    parser.add_argument("--always-set-cfg",
+        choices=["yes", "no"], default=None,
+        dest="always_setcfg",
+        help="on 'yes', all subsequent 'devpi use' will implicitely use "
+             "--set-cfg.  The setting is stored with the devpi client "
+             "config file and can be cleared with '--always-set-cfg=no'.")
     parser.add_argument("--venv", action="store", default=None,
         help="set virtual environment to use for install activities. "
              "specify '-' to unset it.")
@@ -460,18 +472,24 @@ def getjson(parser):
 def list_(parser):
     """ list project versions and files for the current index.
 
-    Without a ``spec`` argument this command will show the names
-    of all projects that have been used.  With a spec argument
-    it will show all release files.  RED files come from an
-    an inherited version which is shadowed by an inheriting index.
+    Without a spec argument this command will show the names
+    of all projects which have releases on the current index.
+    You can use a pip/setuptools style spec argument to show files
+    for particular versions of a project.
+    RED files come from an an inherited version which is
+    shadowed by an inheriting index.
     """
     parser.add_argument("-f", "--failures", action="store_true",
         dest="failures",
         help="show test setup/failure logs")
+
+    parser.add_argument("--all", action="store_true",
+        help="show all versions instead of just the newest")
+
     parser.add_argument("spec", nargs="?",
-        help="show only info for a project/version/release file.  "
-             "Example specs: 'pytest' or 'pytest-2.3.5' or "
-             "'pytest-2.3.5.tar.gz'")
+        help="show info for a project or a specific release. "
+             "Example specs: pytest or 'pytest>=2.3.5'"
+             " (Quotes are needed to prevent shell redirection)")
 
 @subcommand("devpi.list_remove:main_remove")
 def remove(parser):
@@ -484,7 +502,7 @@ def remove(parser):
     parser.add_argument("spec",
         help="remove info/files for a project/version/release file from the "
              "current index. "
-             "Example specs: 'pytest' or 'pytest-2.3.5' or 'pytest-2.3.5.tar.gz'")
+             "Example specs: 'pytest' or 'pytest>=2.3.5'")
 
 @subcommand("devpi.user")
 def user(parser):
@@ -556,45 +574,57 @@ def index(parser):
         help="key=value configuration item. Possible key=value are "
              "bases=CSV, volatile=True|False, acl_upload=CSV)")
 
-@subcommand("devpi.upload.upload")
-def upload(parser):
-    """ prepare and upload packages to the current index.
+if sys.platform == "win32":
+    default_format = "sdist.zip"
+else:
+    default_format = "sdist.tgz"
 
-    This command wraps ``setup.py`` invocations to build and
-    upload releases, release files and documentation to your
-    in-use index (see "devpi use").
+@subcommand("devpi.upload")
+def upload(parser):
+    """ (build and) upload packages to the current devpi-server index.
+
+    You can directly upload existing release files by specifying
+    their file system path as positional arguments.  Such release files
+    need to contain package metadata as created by setup.py or
+    wheel invocations.
+
+    Or, if you don't specify any path, a setup.py file must exist
+    and will be used to perform build and upload commands.
     """
-    #parser.add_argument("-l", dest="showstatus",
-    #    action="store_true", default=None,
-    #    help="show remote versions, local version and package types")
     #parser.add_argument("--ver", dest="setversion",
     #    action="store", default=None,
     #    help="fill version string into setup.py, */__init__.py */conf.py files")
     #parser.add_argument("--incver", action="store_true",
     #    help="retrieve max remove version, increment and set it like --ver")
-    parser.add_argument("--no-vcs", action="store_true", dest="novcs",
+    build = parser.add_argument_group("build options")
+
+    build.add_argument("--no-vcs", action="store_true", dest="novcs",
         help="don't VCS-export to a fresh dir, just execute setup.py scripts "
              "directly using their dirname as current dir. ")
 
-    parser.add_argument("--formats", default="sdist.tgz", action="store",
+    build.add_argument("--formats", default=default_format, action="store",
         help="comma separated list of build formats (passed to setup.py). "
              "Examples sdist.zip,bdist_egg,bdist_wheel,bdist_dumb.")
-    parser.add_argument("--from-dir", action="store", default=None,
-        dest="fromdir",
-        help="upload all archive files from the specified directory")
-    parser.add_argument("--only-latest", action="store_true",
-        help="upload only latest version if multiple archives for a "
-             "package are found (only effective with --from-dir)")
-    parser.add_argument("--dry-run", dest="dryrun",
-        action="store_true", default=None,
-        help="don't perform any server-modifying actions")
-    parser.add_argument("--with-docs", action="store_true", default=None,
+    build.add_argument("--with-docs", action="store_true", default=None,
         dest="withdocs",
         help="build sphinx docs and upload them to index. "
-             "this triggers 'setup.py build_sphinx ... upload_docs ...'")
-    parser.add_argument("--only-docs", action="store_true", default=None,
+             "this triggers 'setup.py build_sphinx' for building")
+    build.add_argument("--only-docs", action="store_true", default=None,
         dest="onlydocs",
-        help="as --with-docs but don't upload release files")
+        help="as --with-docs but don't build or upload release files")
+
+    direct = parser.add_argument_group("direct file upload options")
+    direct.add_argument("--from-dir", action="store_true", default=None,
+        dest="fromdir",
+        help="recursively look for archive files in path if it is a dir")
+    direct.add_argument("--only-latest", action="store_true",
+        help="upload only latest version if multiple archives for a "
+             "package are found (only effective with --from-dir)")
+    direct.add_argument("--dry-run", dest="dryrun",
+        action="store_true", default=None,
+        help="don't perform any server-modifying actions")
+    direct.add_argument("path", action="store", nargs="*",
+        help="path to archive file to be inspected and uploaded.")
     #parser.add_argument("-y", dest="yes",
     #    action="store_true", default=None,
     #    help="answer yes on interactive questions. ")
@@ -607,17 +637,28 @@ def test(parser):
     Download a package and run tests as configured by the
     tox.ini file (which must be contained in the package).
     """
-    parser.add_argument("-e", metavar="VENV", type=str, dest="venv",
+    parser.add_argument("-e", metavar="ENVNAME", type=str, dest="venv",
         default=None, action="store",
-        help="virtual environment to run from the tox.ini")
+        help="tox test environment to run from the tox.ini")
+
+    parser.add_argument("-c", metavar="PATH", type=str, dest="toxini",
+        default=None, action="store",
+        help="tox configuration file to use with unpacked package")
+
+    parser.add_argument("--fallback-ini", metavar="PATH", type=str,
+        dest="fallback_ini",
+        default=None, action="store",
+        help="tox ini file to be used if the downloaded package has none")
+
+    parser.add_argument("--tox-args", metavar="toxargs", action="store",
+        dest="toxargs", default=None,
+        help="extra command line arguments for tox. e.g. "
+             "--toxargs=\"-c othertox.ini\"")
 
     parser.add_argument("pkgspec", metavar="pkgspec", type=str,
         default=None, action="store", nargs=1,
         help="package specification to download and test")
 
-    parser.add_argument("--tox-args", metavar="toxargs", action="store",
-        dest="toxargs", default=None,
-        help="extra command line arguments for tox")
 
 @subcommand("devpi.push")
 def push(parser):
@@ -670,7 +711,7 @@ def verify_reply_version(hub, reply):
         if not hasattr(hub, "_WARNAPI_DELIVERED"):
             hub.error("WARN: devpi-client-%s got an unversioned reply, "
                       "assuming API-VERSION 1 (as implemented by "
-                      "devpi-server-1.1)" % client_version)
+                      "devpi-server-1.1 and 1.2)" % client_version)
             hub._WARNAPI_DELIVERED = True
         return
     if version in acceptable_api_version:
