@@ -1,12 +1,12 @@
 import os
 import sys
 import py
+import re
 
 import json
 
-from devpi import log, cached_property
-from devpi.util import url as urlutil
-import posixpath
+from devpi import log
+from devpi_common.url import URL
 
 if sys.platform == "win32":
     vbin = "Scripts"
@@ -22,6 +22,18 @@ def currentproperty(name):
         self._currentdict[name] = val
     return property(propget, propset)
 
+def urlproperty(name):
+    def propget(self):
+        val = self._currentdict.get(name, None)
+        if val:
+            return URL(val)
+
+    def propset(self, val):
+        if isinstance(val, URL):
+            val = val.url
+        self._currentdict[name] = val
+    return property(propget, propset)
+
 class Current(object):
     index = currentproperty("index")
     simpleindex = currentproperty("simpleindex")
@@ -30,16 +42,20 @@ class Current(object):
     resultlog = currentproperty("resultlog")
     venvdir = currentproperty("venvdir")
     _auth = currentproperty("auth")
+    always_setcfg = currentproperty("always_setcfg")
+
+    @property
+    def index_url(self):
+        if self.index:
+            return URL(self.index)
+        return URL("")
 
     def __init__(self, path):
         self.path = path
-        self._setupcurrentdict()
-
-    def _setupcurrentdict(self):
-        self._currentdict = d = {}
+        self._currentdict = {}
         if self.path.check():
             log.debug("loading current from %s" % self.path)
-            d.update(json.loads(self.path.read()))
+            self._currentdict.update(json.loads(self.path.read()))
         else:
             log.debug("no client config found at %s" % self.path)
 
@@ -93,27 +109,25 @@ class Current(object):
         return self.path and self.path.check()
 
     def _normalize_url(self, url):
-        url = url.rstrip("/") + "/"
-        if not urlutil.ishttp(url):
-            base = urlutil.getnetloc(self.simpleindex, scheme=True)
-            url = urlutil.joinpath(base, url)
+        url = URL(url, asdir=1)
+        if not url.is_valid_http_url():
+            url = URL(self.simpleindex, url.url).url
         return url
 
     def configure_fromurl(self, hub, url):
         url = self.get_index_url(url)
-        if not urlutil.is_valid_url(url):
-            hub.fatal("invalid URL: %s" % url)
-        r = hub.http_api("get", url.rstrip("/") + "/+api", quiet=True)
-        self._configure_from_server_api(r["result"], url)
+        if not url.is_valid_http_url():
+            hub.fatal("invalid URL: %s" % url.url)
+        r = hub.http_api("get", url.addpath("+api"), quiet=True)
+        self._configure_from_server_api(r.result, url)
 
     def _configure_from_server_api(self, result, url):
-        rooturl = urlutil.getnetloc(url, scheme=True) + "/"
+        rooturl = url.joinpath("/")
         data = {}
-        url_keys = set(devpi_endpoints)
-        for name in url_keys:
+        for name in devpi_endpoints:
             val = result.get(name, None)
             if val is not None:
-                val = urlutil.joinpath(rooturl, val)
+                val = rooturl.joinpath(val).url
             data[name] = val
         self.reconfigure(data)
         status = result.get("authstatus", None)
@@ -134,14 +148,14 @@ class Current(object):
     @property
     def rooturl(self):
         if self.login:
-            return urlutil.joinpath(self.login, "/")
+            return URL(self.login, "/").url
 
     def get_user_url(self, user=None):
         if user is None:
             user = self.get_auth_user()
             if not user:
                 raise ValueError("no current authenticated user")
-        return urlutil.joinpath(self.rooturl, user)
+        return URL(self.rooturl).addpath(user)
 
     def get_index_url(self, indexname=None, slash=True):
         if indexname is None:
@@ -149,18 +163,13 @@ class Current(object):
             if indexname is None:
                 raise ValueError("no index name")
         if "/" not in indexname:
-            userurl = self.get_user_url()
-            return urlutil.joinpath(userurl + "/", indexname)
-        url = urlutil.joinpath(self.rooturl, indexname)
-        url = url.rstrip("/")
-        if slash:
-            url = url.rstrip("/") + "/"
-        return url
+            return self.get_user_url().addpath(indexname)
+        if not slash:
+            indexname = indexname.rstrip("/")
+        return URL(self.rooturl).joinpath(indexname, asdir=slash)
 
     def get_project_url(self, name):
-        baseurl = self.get_index_url(slash=True)
-        url = urlutil.joinpath(baseurl, name) + "/"
-        return url
+        return self.index_url.addpath(name, asdir=1)
 
 def out_index_list(hub, data):
     for user in data:
@@ -213,7 +222,7 @@ def main(hub, args=None):
         if not hub.current.rooturl:
             hub.fatal("not connected to any server")
         r = hub.http_api("GET", hub.current.rooturl, {}, quiet=True)
-        out_index_list(hub, r["result"])
+        out_index_list(hub, r.result)
         return 0
 
     showurls = args.urls or args.debug
@@ -229,8 +238,8 @@ def main(hub, args=None):
                 for name in devpi_endpoints:
                     hub.info("%16s: %s" %(name, getattr(current, name)))
             else:
-                hub.info("using index: %s (%s)" % (current.index, login_status))
-        elif current.rooturl:
+                hub.info("current devpi index: %s (%s)" % (current.index, login_status))
+        else:
             hub.info("using server: %s (%s)" % (current.rooturl, login_status))
             hub.error("no current index: type 'devpi use -l' "
                       "to discover indices")
@@ -241,7 +250,101 @@ def main(hub, args=None):
         hub.info("venv for install command: %s" % current.venvdir)
     #else:
     #    hub.line("no current install venv set")
+    if hub.args.always_setcfg:
+        always_setcfg = hub.args.always_setcfg == "yes"
+        hub.current.reconfigure(dict(always_setcfg=always_setcfg))
+    if hub.args.setcfg or hub.current.always_setcfg:
+        if not hub.current.index:
+            hub.error("no index configured: cannot set pip/easy_install index")
+        else:
+            indexserver = hub.current.simpleindex
+            DistutilsCfg().write_indexserver(indexserver)
+            PipCfg().write_indexserver(indexserver)
 
+    show_one_conf(hub, DistutilsCfg())
+    show_one_conf(hub, PipCfg())
+    hub.line("always-set-cfg: %s" % ("yes" if hub.current.always_setcfg else
+                                     "no"))
+
+def show_one_conf(hub, cfg):
+    if not cfg.exists():
+        status = "no config file exists"
+    elif not cfg.indexserver:
+        status = "no index server configured"
+    else:
+        status = cfg.indexserver
+    hub.info("%-19s: %s" %(cfg.screen_name, status))
+
+class BaseCfg:
+    config_name = "index_url"
+    regex = re.compile(r"(index[_-]url)\s*=\s*(.*)")
+
+    def __init__(self, path=None):
+        if path is None:
+            path = self.default_location
+        self.screen_name = str(path)
+        self.path = py.path.local(path, expanduser=True)
+        self.backup_path = self.path + "-bak"
+
+    def exists(self):
+        return self.path.exists()
+
+    @property
+    def indexserver(self):
+        if self.path.exists():
+            for line in self.path.readlines(cr=0):
+                m = self.regex.match(line)
+                if m:
+                    return m.group(2)
+
+    def write_default(self, indexserver):
+        if self.path.exists():
+            raise ValueError("config file already exists")
+        content = "\n".join([self.section_name,
+                             "%s = %s\n" % (self.config_name, indexserver)])
+        self.path.ensure().write(content)
+
+    def write_indexserver(self, indexserver):
+        self.ensure_backup_file()
+        if not self.path.exists():
+            self.write_default(indexserver)
+        else:
+            if self.indexserver:
+                section = None
+            else:
+                section = self.section_name
+            newlines = []
+            found = False
+            for line in self.path.readlines(cr=1):
+                if not section:
+                    m = self.regex.match(line)
+                    if m:
+                        line = "%s = %s\n" % (m.group(1), indexserver)
+                        found = True
+                else:
+                    if section in line.lower():
+                        line = line + "%s = %s\n" %(
+                                                self.config_name, indexserver)
+                        found = True
+                newlines.append(line)
+            if not found:
+                newlines.append(self.section_name + "\n")
+                newlines.append("%s = %s\n" %(self.config_name, indexserver))
+            self.path.write("".join(newlines))
+
+    def ensure_backup_file(self):
+        if self.path.exists() and not self.backup_path.exists():
+             self.path.copy(self.backup_path)
+
+class DistutilsCfg(BaseCfg):
+    section_name = "[easy_install]"
+    default_location = ("~/.pydistutils.cfg" if sys.platform != "win32"
+                        else "~/pydistutils.cfg")
+
+class PipCfg(BaseCfg):
+    section_name = "[global]"
+    default_location = ("~/.pip/pip.conf" if sys.platform != "win32"
+                        else "~/pip/pip.ini")
 
 
 def parse_keyvalue_spec(keyvaluelist, keyset=None):
