@@ -1,9 +1,4 @@
-import os
-import sys
 import py
-import json
-
-from devpi import log
 from devpi.use import parse_keyvalue_spec
 
 def getnewpass(hub, username):
@@ -18,11 +13,12 @@ def getnewpass(hub, username):
 def user_create(hub, user, kvdict):
     if "password" not in kvdict:
         kvdict["password"] = getnewpass(hub, user)
-    res = hub.http_api("put", hub.current.get_user_url(user), kvdict)
+    hub.http_api("put", hub.current.get_user_url(user), kvdict)
     hub.info("user created: %s" % user)
 
 def user_modify(hub, user, kvdict):
-    hub.http_api("patch", hub.current.get_user_url(user), kvdict)
+    reply = hub.http_api("patch", hub.current.get_user_url(user), kvdict)
+    hub.info("user modified: %s" % user)
 
 def user_delete(hub, user):
     hub.http_api("delete", hub.current.get_user_url(user), None)
@@ -30,8 +26,7 @@ def user_delete(hub, user):
 
 def user_list(hub):
     r = hub.http_api("get", hub.current.rooturl)
-    userdict = r["result"]
-    for name in userdict or []:
+    for name in r.result or []:
         hub.line(name)
 
 def main(hub, args):
