@@ -1,13 +1,13 @@
 
 # content of conftest.py
-
+import os
 import random
 import pytest
 import textwrap
 import py
 import sys
-import os
 import json
+from devpi import log
 
 from _pytest.pytester import RunResult, LineMatcher
 from devpi.main import Hub, initmain, parse_args
@@ -19,25 +19,6 @@ std = py.std
 
 pytest_plugins = "pytester"
 
-from devpi.util import url as urlutil
-
-def pytest_runtest_makereport(item, call):
-    if "incremental" in item.keywords:
-        if call.excinfo is not None:
-            item.failed = True
-
-def pytest_runtest_teardown(item, nextitem):
-    if nextitem and getattr(item, "failed", False):
-        nextitem.previousfailed = item
-
-def pytest_runtest_setup(item):
-    if "incremental" in item.keywords:
-        lastitem = getattr(item, "previousfailed", None)
-        if lastitem is not None:
-            pytest.xfail("previous test failed (%s)" %lastitem.name)
-    #else:
-    #    assert 0
-
 def pytest_addoption(parser):
     parser.addoption("--fast", help="skip functional/slow tests", default=False,
                      action="store_true")
@@ -53,7 +34,7 @@ class PopenFactory:
         self.addfinalizer = addfinalizer
 
     def __call__(self, args, pipe=False, **kwargs):
-        args = map(str, args)
+        args = [str(x) for x in args]
         if pipe:
             print ("$ %s [piped]" %(" ".join(args),))
             popen = gsub.Popen(args, stdout=gsub.PIPE, stderr=gsub.STDOUT)
@@ -124,6 +105,7 @@ def port_of_liveserver(request):
     port = random.randint(2001, 64000)
     clientdir = request.config._tmpdirhandler.mktemp("liveserver")
     subprocess.check_call(["devpi-server", "--serverdir", str(clientdir),
+                           "--debug",
                              "--port", str(port), "--start"])
     def stop():
         subprocess.check_call(["devpi-server", "--serverdir", str(clientdir),
@@ -152,15 +134,16 @@ def create_and_upload(request, devpi, initproj, Popen):
     def upload(name, filedefs=None):
         initproj(name, filedefs)
         # we need to patch .pypirc
-        with devpi.patched_pypirc as reponame:
-            popen = Popen([sys.executable, "setup.py",
-                           "register", "-r", reponame])
-            popen.communicate()
-            assert popen.returncode == 0
-            popen = Popen([sys.executable, "setup.py", "sdist", "upload",
-                           "-r", reponame])
-            popen.communicate()
-            assert popen.returncode == 0
+        #with devpi.patched_pypirc as reponame:
+        #    popen = Popen([sys.executable, "setup.py",
+        #                   "register", "-r", reponame])
+        #    popen.communicate()
+        #    assert popen.returncode == 0
+        #    popen = Popen([sys.executable, "setup.py", "sdist", "upload",
+        #                   "-r", reponame])
+        #    popen.communicate()
+        #    assert popen.returncode == 0
+        devpi("upload")
     return upload
 
 
@@ -254,7 +237,6 @@ def pytest_runtest_makereport(__multicall__, item, call):
 def ext_devpi(request, tmpdir, devpi):
     def doit(*args, **kwargs):
         tmpdir.chdir()
-        clientdir = devpi.clientdir
         result = runprocess(tmpdir,
             ["devpi", "--clientdir", devpi.clientdir] + list(args))
         ret = kwargs.get("ret", 0)
@@ -294,9 +276,8 @@ def cmd_devpi(tmpdir):
         print_info("*** inline$ %s" % " ".join(callargs))
         hub, method = initmain(callargs)
         try:
-            ret = method(hub, hub.args)
+            method(hub, hub.args)
         except SystemExit as sysex:
-            ret = sysex.args[0] or 1
             hub.sysex = sysex
         expected = kwargs.get("code", None)
         if expected is not None:
@@ -328,11 +309,6 @@ def runprocess(tmpdir, cmdargs):
     return RunResult(ret, outerr, None, std.time.time()-now)
 
 
-import pytest
-from mock import Mock
-import devpi.util.url as urlutil
-from devpi import log
-
 @pytest.fixture
 def mockhtml(monkeypatch):
     def mockhtml(cache, mockurl, content):
@@ -366,11 +342,7 @@ def create_venv(request, testdir, monkeypatch):
             bindir = "Scripts"
         else:
             bindir = "bin"
-        ac = venvdir.join(bindir, "activate_this.py")
-        assert ac.check(), ac
-        #print ac, os.environ["PATH"]
-        execfile(str(ac), dict(__file__=str(ac)))
-        #print ac, os.environ["PATH"]
+        monkeypatch.setenv("PATH", bindir + os.pathsep + os.environ["PATH"])
         return venvdir
     return do_create_venv
 
@@ -393,6 +365,7 @@ def loghub(tmpdir, mock_http_api):
 def makehub(request):
     handler = request.config._tmpdirhandler
     def mkhub(arglist):
+        arglist = [str(x) for x in arglist]
         tmp = handler.mktemp("hub")
         for x in arglist:
             if "--clientdir" in x:
@@ -400,11 +373,8 @@ def makehub(request):
         else:
             arglist.append("--clientdir=%s" % tmp)
         args = parse_args(["devpi_"] + arglist)
-        old = tmp.chdir()
-        try:
+        with tmp.as_cwd():
             return Hub(args)
-        finally:
-            old.chdir()
     return mkhub
 
 @pytest.fixture
@@ -421,7 +391,8 @@ def mock_http_api(monkeypatch):
             self.called = []
             self._json_responses = {}
 
-        def __call__(self, method, url, kvdict=None, quiet=False, auth=None):
+        def __call__(self, method, url, kvdict=None, quiet=False, auth=None,
+                     fatal=True):
             self.called.append((method, url, kvdict))
             reply_data = self._json_responses.get(url)
             if reply_data is not None:
