@@ -1,10 +1,6 @@
+#from __future__ import unicode_literals
 
-import urllib
 import pytest
-import py
-from devpi import use
-from devpi import log
-from devpi.main import Hub, parse_args
 from devpi.use import *
 
 def test_ask_confirm(makehub, monkeypatch):
@@ -41,11 +37,29 @@ class TestUnit:
         assert newcurrent.venvdir == current.venvdir
         assert newcurrent.login == current.login
 
+    def test_write_and_read_always_setcfg(self, tmpdir):
+        path=tmpdir.join("current")
+        current = Current(path)
+        assert not current.simpleindex
+        current.reconfigure(dict(
+                pypisubmit="/post",
+                simpleindex="/index",
+                login="/login",
+                resultlog="/"))
+        assert current.simpleindex
+        current.reconfigure(dict(always_setcfg=True))
+        newcurrent = Current(path)
+        assert newcurrent.always_setcfg == True
+        newcurrent.reconfigure(data=dict(simpleindex="/index2"))
+        current = Current(path)
+        assert current.always_setcfg
+        assert current.simpleindex == "/index2"
+
     def test_normalize_url(self, tmpdir):
         current = Current(tmpdir.join("current"))
         current.reconfigure(dict(simpleindex="http://my.serv/index1"))
         url = current._normalize_url("index2")
-        assert url == "http://my.serv/index2/"
+        assert url == "http://my.serv/index2"
 
     def test_auth_multisite(self, tmpdir):
         current = Current(tmpdir.join("current"))
@@ -64,7 +78,7 @@ class TestUnit:
     def test_invalid_url(self, loghub, tmpdir):
         current = Current(tmpdir.join("current"))
         with pytest.raises(SystemExit):
-            current.configure_fromurl(loghub, "http://heise.de:/qwe")
+            current.configure_fromurl(loghub, "http://heise.de:1802:31/qwe")
 
     def test_auth_handling(self, loghub, tmpdir):
         current = Current(tmpdir.join("current"))
@@ -79,29 +93,28 @@ class TestUnit:
 
         # ok response
         d["authstatus"] = ["ok", "user"]
-        current._configure_from_server_api(d, current.rooturl)
+        current._configure_from_server_api(d, URL(current.rooturl))
         assert current.get_auth() == ("user", "password")
 
         # invalidation response
         d["authstatus"] = ["nouser", "user"]
-        current._configure_from_server_api(d, current.rooturl)
+        current._configure_from_server_api(d, URL(current.rooturl))
         assert not current.get_auth()
 
     def test_use_with_no_rooturl(self, capfd, cmd_devpi, monkeypatch):
         from devpi import main
         monkeypatch.setattr(main.Hub, "http_api", None)
-        hub = cmd_devpi("use", "some/index", code=None)
+        cmd_devpi("use", "some/index", code=None)
         out, err = capfd.readouterr()
         assert "invalid" in out
 
     def test_use_with_nonexistent_domain(self, capfd, cmd_devpi, monkeypatch):
-        from devpi import main
         from requests.sessions import Session
         from requests.exceptions import ConnectionError
         def raise_connectionerror(*args, **kwargs):
             raise ConnectionError("qwe")
         monkeypatch.setattr(Session, "request", raise_connectionerror)
-        hub = cmd_devpi("use", "http://qlwkejqlwke", code=-1)
+        cmd_devpi("use", "http://qlwkejqlwke", code=-1)
         out, err = capfd.readouterr()
         assert "could not connect" in out
 
@@ -167,7 +180,7 @@ class TestUnit:
                         authstatus=["noauth", ""],
                    ))
 
-        hub = cmd_devpi("use", "http://world/")
+        cmd_devpi("use", "http://world/")
         mock_http_api.set("http://world/", 200, result=dict(
             user1=dict(indexes={"dev": {"bases": ["x"],
                 "volatile": False}})
@@ -194,6 +207,33 @@ class TestUnit:
         hub = cmd_devpi("use", "--venv=%s" % venvdir.basename)
         assert hub.current.venvdir == venvdir
 
+    def test_main_setcfg(self, mock_http_api, cmd_devpi, tmpdir, monkeypatch):
+        monkeypatch.setattr(PipCfg, "default_location", tmpdir.join("pip.cfg"))
+        monkeypatch.setattr(DistutilsCfg, "default_location",
+                            tmpdir.join("dist.cfg"))
+        mock_http_api.set("http://world/+api", 200,
+                    result=dict(
+                        pypisubmit="",
+                        simpleindex="/simple",
+                        resultlog="/resultlog/",
+                        index="/",
+                        bases="",
+                        login="/+login/",
+                        authstatus=["noauth", ""],
+                   ))
+
+        hub = cmd_devpi("use", "--set-cfg", "http://world")
+        assert PipCfg.default_location.exists()
+        content = PipCfg.default_location.read()
+        assert "index_url = http://world/" in content
+        assert DistutilsCfg.default_location.exists()
+        content = DistutilsCfg.default_location.read()
+        assert "index_url = http://world/" in content
+        hub = cmd_devpi("use", "--always-set-cfg=yes")
+        assert hub.current.always_setcfg
+        hub = cmd_devpi("use", "--always-set-cfg=no")
+        assert not hub.current.always_setcfg
+
 
 @pytest.mark.parametrize("input expected".split(), [
     (["hello=123", "world=42"], dict(hello="123", world="42")),
@@ -208,3 +248,60 @@ def test_parse_keyvalue_spec_unknown_key():
 
 def test_user_no_index(loghub):
     out_index_list(loghub, {"user": {"username": "user"}})
+
+class TestCfgParsing:
+    @pytest.fixture(scope="class", params=[DistutilsCfg, PipCfg])
+    def cfgclass(self, request):
+        return request.param
+
+    def test_empty(self, cfgclass, tmpdir):
+        p = tmpdir.join("cfg")
+        assert not cfgclass(p).exists()
+        assert cfgclass(p).indexserver is None
+        assert cfgclass(p).screen_name == str(p)
+        assert cfgclass.default_location
+
+    def test_read(self, cfgclass, tmpdir):
+        p = tmpdir.join("cfg")
+        cfg = cfgclass(p)
+        cfg.write_default("http://some.com/something")
+        with pytest.raises(ValueError):
+            cfg.write_default("http://some.com/something")
+        cfg = cfgclass(p)
+        assert cfg.exists()
+        assert cfg.indexserver == "http://some.com/something"
+        assert cfg.screen_name == str(p)
+
+    def test_read_config_but_no_index(self, tmpdir, cfgclass):
+        path = tmpdir.join("cfg")
+        path.write(cfgclass.section_name + "\n")
+        cfg = cfgclass(path)
+        cfg.write_indexserver("http://qwe")
+        cfg = cfgclass(path)
+        assert cfg.indexserver == "http://qwe"
+
+    def test_read_config_but_no_section(self, tmpdir, cfgclass):
+        path = tmpdir.join("cfg")
+        path.write("")
+        cfg = cfgclass(path)
+        cfg.write_indexserver("http://qwe")
+        cfg = cfgclass(path)
+        assert cfg.indexserver == "http://qwe"
+
+    def test_write_fresh(self, cfgclass, tmpdir):
+        p = tmpdir.join("cfg")
+        cfg = cfgclass(p)
+        cfg.write_indexserver("http://hello.com")
+        assert cfg.indexserver == "http://hello.com"
+
+    def test_rewrite(self, cfgclass, tmpdir):
+        p = tmpdir.join("cfg")
+        cfgclass(p).write_default("http://some.com/something")
+        cfg = cfgclass(p)
+        cfg.write_indexserver("http://hello.com")
+        assert cfg.indexserver == "http://hello.com"
+        assert cfgclass(cfg.backup_path).indexserver == \
+               "http://some.com/something"
+        cfg.write_indexserver("http://hello.com")
+        assert cfgclass(cfg.backup_path).indexserver == \
+               "http://some.com/something"
